package awais.instagrabber.asyncs;

import android.os.AsyncTask;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONObject;

import java.net.HttpURLConnection;
import java.net.URL;

import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.models.ViewerPostModel;
import awais.instagrabber.utils.Utils;

public class PostFetcher extends AsyncTask<Void, Void, ViewerPostModel[]> {
    private final String shortCode;
    private final FetchListener<ViewerPostModel[]> fetchListener;

    public PostFetcher(final String shortCode, final FetchListener<ViewerPostModel[]> fetchListener) {
        this.shortCode = shortCode;
        this.fetchListener = fetchListener;
    }

    @Override
    protected ViewerPostModel[] doInBackground(final Void... voids) {
        try {
            final HttpURLConnection conn = (HttpURLConnection) new URL("https://www.instagram.com/p/" + shortCode + "/?__a=1").openConnection();
            conn.setUseCaches(false);
            conn.connect();

            final int responseCode = conn.getResponseCode();
            if (responseCode == 200 || responseCode == 201) {
                final JSONObject media = new JSONObject(Utils.readFromConnection(conn)).getJSONObject("graphql")
                        .getJSONObject("shortcode_media");

                final JSONArray captions = media.getJSONObject("edge_media_to_caption").getJSONArray("edges");
                final boolean isVideo = media.has("is_video") && media.getBoolean("is_video");

                final String postCaption = captions.length() > 0 ? captions.getJSONObject(0).getJSONObject("node").getString("text") : null;
                final long timestamp = media.getLong("taken_at_timestamp");

                final String username = media.has("owner") ? media.getJSONObject("owner").getString("username") : null;

                if (!media.has("edge_sidecar_to_children"))
                    return new ViewerPostModel[]{new ViewerPostModel(isVideo, media.getString("id"),
                            isVideo ? media.getString("video_url") : media.getString("display_url"),
                            shortCode, postCaption, username,
                            isVideo && media.has("video_view_count") ? media.getLong("video_view_count") : -1,
                            timestamp)};

                final JSONArray children = media.getJSONObject("edge_sidecar_to_children").getJSONArray("edges");
                final ViewerPostModel[] postModels = new ViewerPostModel[children.length()];
                for (int i = 0; i < postModels.length; ++i) {
                    final JSONObject node = children.getJSONObject(i).getJSONObject("node");
                    final boolean isChildVideo = node.getBoolean("is_video");
                    postModels[i] = new ViewerPostModel(isChildVideo, node.getString("id"),
                            isChildVideo ? node.getString("video_url") : node.getString("display_url"),
                            node.getString("shortcode"), postCaption, username,
                            isChildVideo && node.has("video_view_count") ? node.getLong("video_view_count") : -1,
                            timestamp);
                    postModels[i].sliderDisplayUrl = node.getString("display_url");
                }
                return postModels;

            }
        } catch (Exception e) {
            Log.e("AWAISKING_APP", "", e);
        }
        return null;
    }

    @Override
    protected void onPostExecute(final ViewerPostModel[] postModels) {
        if (fetchListener != null) fetchListener.onResult(postModels);
    }
}
