package awais.instagrabber.utils;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapShader;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Outline;
import android.graphics.Paint;
import android.graphics.Shader;
import android.graphics.drawable.BitmapDrawable;
import android.os.Build;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewOutlineProvider;

import androidx.appcompat.widget.AppCompatImageView;

public class CircularImageView extends AppCompatImageView {
    private int viewWidth, viewHeight;
    private final int borderWidth = 6;
    private int color = Color.TRANSPARENT;
    private final Paint paint = new Paint();
    private final Paint paintBorder = new Paint();
    private BitmapShader shader;
    private Bitmap bitmap;

    public CircularImageView(final Context context) {
        super(context);
        setup();
    }

    public CircularImageView(final Context context, final AttributeSet attrs) {
        super(context, attrs);
        setup();
    }

    public CircularImageView(final Context context, final AttributeSet attrs, final int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        setup();
    }

    private void setup() {
        paint.setAntiAlias(true);
        paintBorder.setColor(color);
        paintBorder.setAntiAlias(true);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            setOutlineProvider(new ViewOutlineProvider() {
                @Override
                public void getOutline(final View view, final Outline outline) {
                    outline.setRoundRect(borderWidth + 4, borderWidth + 4, viewWidth, viewHeight, viewHeight >> 1);
                }
            });
        }
    }

    @SuppressLint("CanvasSize")
    @Override
    public void onDraw(final Canvas canvas) {
        final BitmapDrawable bitmapDrawable = (BitmapDrawable) getDrawable();
        if (bitmapDrawable != null) {
            final Bitmap bmp = bitmapDrawable.getBitmap();
            final boolean changed = bmp != bitmap;
            if (changed && bitmap != null && !bitmap.isRecycled()) bitmap.recycle();
            bitmap = bmp;
            if (bitmap != null) {
                if (shader == null || changed) {
                    shader = new BitmapShader(Bitmap.createScaledBitmap(bitmap, canvas.getWidth(), canvas.getHeight(), true),
                            Shader.TileMode.CLAMP, Shader.TileMode.CLAMP);
                    paint.setShader(shader);
                }
                if (changed) color = 0;
                paintBorder.setColor(color);
                final int circleCenter = viewWidth / 2;
                canvas.drawCircle(circleCenter + borderWidth, circleCenter + borderWidth, circleCenter + borderWidth - 4.0f, paintBorder);
                canvas.drawCircle(circleCenter + borderWidth, circleCenter + borderWidth, circleCenter - 4.0f, paint);
            }
        }
    }

    @Override
    protected void onMeasure(final int widthMeasureSpec, final int heightMeasureSpec) {
        final int width = MeasureSpec.getMode(widthMeasureSpec) == MeasureSpec.EXACTLY ? MeasureSpec.getSize(widthMeasureSpec) : viewWidth;
        final int height = (MeasureSpec.getMode(heightMeasureSpec) == MeasureSpec.EXACTLY ? MeasureSpec.getSize(heightMeasureSpec) : viewHeight) + 2;

        final int doubleBorder = borderWidth * 2;
        viewWidth = width - doubleBorder;
        viewHeight = height - doubleBorder;

        setMeasuredDimension(width, height);
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        setLayerType(LAYER_TYPE_HARDWARE, null);
    }

    @Override
    protected void onDetachedFromWindow() {
        setLayerType(LAYER_TYPE_NONE, null);
        super.onDetachedFromWindow();
    }

    public void setStoriesBorder() {
        this.color = Color.GREEN;
        invalidate();
    }
}