package awais.instagrabber.utils;

import android.content.Context;
import android.util.TypedValue;

import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

public class GridAutofitLayoutManager extends GridLayoutManager {
    private int mColumnWidth;
    private boolean mColumnWidthChanged = true;

    public GridAutofitLayoutManager(Context context, int columnWidth) {
        super(context, 1);
        if (columnWidth <= 0)
            columnWidth = (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 48, Utils.displayMetrics);
        if (columnWidth <= 0 || columnWidth == mColumnWidth) return;
        mColumnWidth = columnWidth;
        mColumnWidthChanged = true;
    }

    @Override
    public void onLayoutChildren(RecyclerView.Recycler recycler, RecyclerView.State state) {
        final int width = getWidth();
        final int height = getHeight();
        if (mColumnWidthChanged && mColumnWidth > 0 && width > 0 && height > 0) {
            final int totalSpace = getOrientation() == VERTICAL ? width - getPaddingRight() - getPaddingLeft()
                    : height - getPaddingTop() - getPaddingBottom();

            setSpanCount(Math.max(1, totalSpace / mColumnWidth));

            mColumnWidthChanged = false;
        }
        super.onLayoutChildren(recycler, state);
    }
}
