package awais.instagrabber.utils;

import android.content.Intent;
import android.graphics.Typeface;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.text.SpannableStringBuilder;
import android.text.style.RelativeSizeSpan;
import android.text.style.StyleSpan;
import android.util.Log;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;

import com.squareup.picasso.Callback;
import com.squareup.picasso.Picasso;

import java.util.Arrays;

import awais.instagrabber.Main;
import awais.instagrabber.PostViewer;
import awais.instagrabber.R;
import awais.instagrabber.adapters.PostsAdapter;
import awais.instagrabber.asyncs.PostsFetcher;
import awais.instagrabber.asyncs.ProfileFetcher;
import awais.instagrabber.asyncs.StoryStatusFetcher;
import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.models.ClipModel;
import awais.instagrabber.models.PostModel;

import static awais.instagrabber.utils.dialogs.SettingConstants.BOTTOM_TOOLBAR;

public class MainHelper {
    private final FetchListener<PostModel[]> listener = new FetchListener<PostModel[]>() {
        @Override
        public void onResult(final PostModel[] result) {
            if (result != null) {
                final int oldSize = Main.allItems.size();
                Main.allItems.addAll(Arrays.asList(result));

                postsAdapter.notifyItemRangeInserted(oldSize, result.length);

                main.toolbar.setTitle(isHashtag ? "Hashtag: " + main.oldQuery :
                        main.profileModel.getUsername() + " (" + Main.allItems.size() + "/" + main.profileModel.getPostCount() + ")");

                final PostModel model = result[result.length - 1];
                if (model != null) {
                    if (model.getEndCursor() == null && main.progressCircular != null) {
                        main.progressCircular.setVisibility(View.GONE);
                        if (!isHashtag) {
                            main.toolbar.setTitle(main.profileModel.getUsername() +
                                    " (" + main.profileModel.getPostCount() + "/" + main.profileModel.getPostCount() + ")");
                            final Handler handler = new Handler();
                            handler.postDelayed(new Runnable() {
                                @Override
                                public void run() {
                                    main.toolbar.setTitle(main.profileModel.getUsername());
                                    handler.removeCallbacks(this);
                                }
                            }, 1017);
                        }
                    }
                    if (model.hasNextPage()) {
                        if (!isHashtag) {
                            currentlyExecuting = new PostsFetcher(main.profileModel.getId(),
                                    model.getEndCursor(), this).execute();
                        } else if (main.progressCircular != null) main.progressCircular.setVisibility(View.GONE);
                        model.setPageCursor(false, null);
                    }
                }
            }
        }
    };
    @NonNull
    private final Main main;
    private final View infoContainer;
    private PostsAdapter postsAdapter;
    private boolean isHashtag;
    private AsyncTask<Void, Void, PostModel[]> currentlyExecuting;

    public MainHelper(@NonNull final Main main) {
        this.main = main;

        this.infoContainer = main.findViewById(R.id.info_container);
        main.mainProfileImage = infoContainer.findViewById(R.id.mainProfileImage);
        main.mainPostCount = infoContainer.findViewById(R.id.mainPostCount);
        main.mainFollowers = infoContainer.findViewById(R.id.mainFollowers);
        main.mainFollowing = infoContainer.findViewById(R.id.mainFollowing);
        main.mainFullName = infoContainer.findViewById(R.id.mainFullName);
        main.mainBiography = infoContainer.findViewById(R.id.mainBiography);
        main.toolbar = main.findViewById(R.id.toolbar);
        main.mainPosts = main.findViewById(R.id.mainPosts);

        main.progressCircular = main.findViewById(R.id.progress_circular);
        main.privatePage = main.findViewById(R.id.privatePage);

        main.setSupportActionBar(main.toolbar);
        if (Main.sharedPreferences.getBoolean(BOTTOM_TOOLBAR, true)) {
            final LinearLayout linearLayout = (LinearLayout) main.toolbar.getParent();
            linearLayout.removeView(main.toolbar);
            linearLayout.addView(main.toolbar, linearLayout.getChildCount());
        }

        main.mainPosts.setLayoutManager(new GridAutofitLayoutManager(main, Utils.convertDpToPx(130)));
        main.mainPosts.addItemDecoration(new GridSpacingItemDecoration(Utils.convertDpToPx(4)));
        postsAdapter = new PostsAdapter(Main.allItems, v -> {
            final Object tag = v.getTag();
            if (tag instanceof PostModel) {
                final PostModel postModel = (PostModel) tag;

                if (postsAdapter.isSelecting) toggleSelection(postModel);
                else main.startActivity(new Intent(main, PostViewer.class).putExtra("index", postModel.getPosition())
                        .putExtra("post", postModel).putExtra("user", main.oldQuery));
            }
        }, v -> { // long click listener
            final Object tag = v.getTag();
            if (tag instanceof PostModel) {
                postsAdapter.isSelecting = true;
                toggleSelection((PostModel) tag);
            }
            return true;
        });
        main.mainPosts.setAdapter(postsAdapter);
    }

    public void onIntent(final Intent intent) {
        if (intent != null) {
            final String action = intent.getAction();
            if (!Utils.isEmpty(action) && !"android.intent.action.MAIN".equals(action)) {
                intent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION | Intent.FLAG_GRANT_READ_URI_PERMISSION);

                boolean error = true;

                String data = null;
                final Bundle extras = intent.getExtras();
                if (extras != null) {
                    final Object extraData = extras.get(Intent.EXTRA_TEXT);
                    if (extraData != null) {
                        error = false;
                        data = extraData.toString();
                    }
                }

                if (error) {
                    final Uri intentData = intent.getData();
                    if (intentData != null) data = intentData.toString();
                }

                if (data != null && !Utils.isEmpty(data)) {
                    final ClipModel model = Utils.stripString(data);
                    if (model != null) {
                        final String text = model.getText();
                        if (model.isPost()) {
                            main.startActivity(new Intent(main, PostViewer.class)
                                    .putExtra("user", main.oldQuery)
                                    .putExtra("post", new PostModel(false, null, null, null,
                                            text, null, 0)));
                            main.finish();
                        } else {
                            main.oldQuery = text;
                            doSearch();
                        }
                    }
                }
            }
        }
    }

    public void doSearch() {
        if (currentlyExecuting != null) {
            try {
                currentlyExecuting.cancel(true);
            } catch (final Exception e) {
                Log.e("AWAISKING_APP", "", e);
            }
        }
        Main.allItems.clear();
        main.selectedItems.clear();
        if (postsAdapter != null) {
            postsAdapter.isSelecting = false;
            postsAdapter.notifyDataSetChanged();
        }
        main.privatePage.setVisibility(View.GONE);
        main.mainProfileImage.setImageBitmap(null);
        main.mainProfileImage.setImageDrawable(null);
        main.mainPostCount.setText(null);
        main.mainFollowers.setText(null);
        main.mainFollowing.setText(null);
        main.mainFullName.setText(null);
        main.mainBiography.setText(null);
        main.mainBiography.setEnabled(false);
        main.mainProfileImage.setEnabled(false);

        if (main.progressCircular != null) main.progressCircular.setVisibility(main.oldQuery == null ? View.GONE : View.VISIBLE);
        if (main.oldQuery == null) {
            main.toolbar.setTitle(R.string.app_name);
            return;
        }

        isHashtag = main.oldQuery.charAt(0) == '#';

        if (isHashtag) {
            if (infoContainer != null) infoContainer.setVisibility(View.GONE);
            currentlyExecuting = new PostsFetcher(main.oldQuery, listener).execute();
        } else {
            if (infoContainer != null) infoContainer.setVisibility(View.VISIBLE);
            new ProfileFetcher(main.oldQuery, profileModel -> {
                main.profileModel = profileModel;

                if (profileModel == null) {
                    Toast.makeText(main, "Error loading profile!", Toast.LENGTH_SHORT).show();
                    main.toolbar.setTitle(R.string.app_name);
                    return;
                }

                if (!Utils.isEmpty(Main.sharedPreferences.getString("cookie", null))) {
                    new StoryStatusFetcher(profileModel.getId(), result -> {
                        main.storyModels = result;
                        if (result != null && result.length > 0)
                            main.mainProfileImage.setStoriesBorder();
                    }).execute();
                }

                Picasso.get().load(profileModel.getSdProfilePic()).into(main.mainProfileImage, new Callback() {
                    @Override
                    public void onSuccess() {
                        main.mainProfileImage.setEnabled(true);
                    }

                    @Override
                    public void onError(final Exception e) {
                        main.mainProfileImage.setEnabled(false);
                    }
                });

                final String postCount = String.valueOf(profileModel.getPostCount());
                SpannableStringBuilder span = new SpannableStringBuilder(postCount + "\nPosts");
                span.setSpan(new RelativeSizeSpan(1.2f), 0, postCount.length(), 0);
                span.setSpan(new StyleSpan(Typeface.BOLD), 0, postCount.length(), 0);
                main.mainPostCount.setText(span);

                final String followersCount = String.valueOf(profileModel.getFollowersCount());
                span = new SpannableStringBuilder(followersCount + "\nFollowers");
                span.setSpan(new RelativeSizeSpan(1.2f), 0, followersCount.length(), 0);
                span.setSpan(new StyleSpan(Typeface.BOLD), 0, followersCount.length(), 0);
                main.mainFollowers.setText(span);

                final String followingCount = String.valueOf(profileModel.getFollowingCount());
                span = new SpannableStringBuilder(followingCount + "\nFollowing");
                span.setSpan(new RelativeSizeSpan(1.2f), 0, followingCount.length(), 0);
                span.setSpan(new StyleSpan(Typeface.BOLD), 0, followingCount.length(), 0);
                main.mainFollowing.setText(span);

                main.mainProfileImage.setEnabled(false);
                main.mainFullName.setText(profileModel.getName());
                main.mainBiography.setText(profileModel.getBiography());
                main.mainBiography.setEnabled(true);

                main.toolbar.setTitle(profileModel.getUsername());

                if (!profileModel.isPrivate()) {
                    if (main.progressCircular != null) main.progressCircular.setVisibility(View.VISIBLE);
                    main.mainPosts.setVisibility(View.VISIBLE);
                    main.privatePage.setVisibility(View.GONE);

                    currentlyExecuting = new PostsFetcher(profileModel.getId(), listener).execute();
                } else {
                    if (main.progressCircular != null) main.progressCircular.setVisibility(View.GONE);
                    main.mainPosts.setVisibility(View.GONE);
                    main.privatePage.setVisibility(View.VISIBLE);
                }
            }).execute();
        }
    }

    private void toggleSelection(final PostModel postModel) {
        if (postModel != null && postsAdapter != null) {
            if (postModel.isSelected()) main.selectedItems.remove(postModel);
            else main.selectedItems.add(postModel);
            postModel.setSelected(!postModel.isSelected());
            notifyAdapter(postModel);
        }
    }

    public void deselectSelection(final PostModel postModel) {
        if (postModel != null && postsAdapter != null) {
            main.selectedItems.remove(postModel);
            postModel.setSelected(false);
            notifyAdapter(postModel);
        }
    }

    private void notifyAdapter(final PostModel postModel) {
        if (main.selectedItems.size() < 1) postsAdapter.isSelecting = false;
        if (postModel.getPosition() < 0) postsAdapter.notifyDataSetChanged();
        else postsAdapter.notifyItemChanged(postModel.getPosition(), postModel);

        if (main.downloadAction != null) main.downloadAction.setVisible(postsAdapter.isSelecting);
    }

    public boolean isSelectionCleared() {
        if (postsAdapter != null && postsAdapter.isSelecting) {
            for (final PostModel postModel : main.selectedItems) postModel.setSelected(false);
            main.selectedItems.clear();
            postsAdapter.isSelecting = false;
            postsAdapter.notifyDataSetChanged();
            if (main.downloadAction != null) main.downloadAction.setVisible(false);
            return false;
        }
        return true;
    }
}
