package awais.instagrabber.utils.dialogs;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatCheckBox;
import androidx.coordinatorlayout.widget.CoordinatorLayout;

import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import awais.instagrabber.Main;
import awais.instagrabber.R;

import static awais.instagrabber.utils.dialogs.SettingConstants.USE_ALTER_PROFILE;

public class ProfileSettingsDialog extends BottomSheetDialogFragment {
    private Activity activity;
    private final BottomSheetBehavior.BottomSheetCallback bottomSheet = new BottomSheetBehavior.BottomSheetCallback() {
        @Override
        public void onStateChanged(@NonNull final View bottomSheet, final int newState) {
            if (newState == BottomSheetBehavior.STATE_HIDDEN) {
                dismiss();
                if (activity != null) activity.recreate();
            }
        }

        @Override
        public void onSlide(@NonNull final View bottomSheet, final float slideOffset) {}
    };

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        final Dialog dialog = super.onCreateDialog(savedInstanceState);

        final Context context = getContext();
        activity = context instanceof Activity ? (Activity) context : getActivity();

        final View contentView = View.inflate(activity, R.layout.profile_settings_dialog, null);

        final AppCompatCheckBox cbAltProfile = contentView.findViewById(R.id.cbAltProfile);
        cbAltProfile.setChecked(Main.sharedPreferences.getBoolean(USE_ALTER_PROFILE, false));
        cbAltProfile.setOnCheckedChangeListener((v, checked) ->
                Main.sharedPreferences.edit().putBoolean(USE_ALTER_PROFILE, checked).apply());
        ((View) cbAltProfile.getParent()).setOnClickListener(v -> cbAltProfile.performClick());

        dialog.setContentView(contentView);

        final CoordinatorLayout.LayoutParams layoutParams = (CoordinatorLayout.LayoutParams)
                ((View) contentView.getParent()).getLayoutParams();
        final CoordinatorLayout.Behavior<?> behavior = layoutParams.getBehavior();
        if (behavior instanceof BottomSheetBehavior) {
            final BottomSheetBehavior<?> sheetBehavior = (BottomSheetBehavior<?>) behavior;
            sheetBehavior.removeBottomSheetCallback(bottomSheet);
            sheetBehavior.addBottomSheetCallback(bottomSheet);
            sheetBehavior.setSaveFlags(BottomSheetBehavior.SAVE_HIDEABLE | BottomSheetBehavior.SAVE_FIT_TO_CONTENTS
                    | BottomSheetBehavior.SAVE_SKIP_COLLAPSED | BottomSheetBehavior.SAVE_PEEK_HEIGHT);
        }
        return dialog;
    }
}