package awais.instagrabber.utils.dialogs;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CompoundButton;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatCheckBox;
import androidx.coordinatorlayout.widget.CoordinatorLayout;

import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import awais.instagrabber.Login;
import awais.instagrabber.Main;
import awais.instagrabber.R;

import static awais.instagrabber.utils.dialogs.SettingConstants.AUTOPLAY_VIDEOS;
import static awais.instagrabber.utils.dialogs.SettingConstants.BOTTOM_SNACK;
import static awais.instagrabber.utils.dialogs.SettingConstants.BOTTOM_TOOLBAR;
import static awais.instagrabber.utils.dialogs.SettingConstants.DOWNLOAD_USER_FOLDER;
import static awais.instagrabber.utils.dialogs.SettingConstants.MUTED_VIDEOS;

public class SettingsDialog extends BottomSheetDialogFragment {
    private Activity activity;
    private final BottomSheetBehavior.BottomSheetCallback bottomSheet = new BottomSheetBehavior.BottomSheetCallback() {
        @Override
        public void onStateChanged(@NonNull final View bottomSheet, final int newState) {
            if (newState == BottomSheetBehavior.STATE_HIDDEN) {
                dismiss();
                if (activity != null) activity.recreate();
            }
        }

        @Override
        public void onSlide(@NonNull final View bottomSheet, final float slideOffset) {}
    };
    private final CompoundButton.OnCheckedChangeListener checkChangeListener = (checkBox, checked) -> {
        final SharedPreferences.Editor editor = Main.sharedPreferences.edit();
        final int id = checkBox.getId();
        if (id == R.id.cbDownloadUsername) editor.putBoolean(DOWNLOAD_USER_FOLDER, checked);
        else if (id == R.id.cbBottomToolbar) editor.putBoolean(BOTTOM_TOOLBAR, checked);
        else if (id == R.id.cbBottomSnack) editor.putBoolean(BOTTOM_SNACK, checked);
        else if (id == R.id.cbAutoplayVideos) editor.putBoolean(AUTOPLAY_VIDEOS, checked);
        else if (id == R.id.cbMuteVideos) editor.putBoolean(MUTED_VIDEOS, checked);
        editor.apply();
    };
    private final View.OnClickListener clickListener = v -> {
        if (v instanceof Button) {
            startActivity(v.getId() == R.id.btnLogin ? new Intent(v.getContext(), Login.class) :
                    new Intent(Intent.ACTION_VIEW, Uri.parse("https://gitlab.com/AwaisKing/instagrabber/")));
        } else ((ViewGroup) v).getChildAt(0).performClick();
    };

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        final Dialog dialog = super.onCreateDialog(savedInstanceState);

        final Context context = getContext();
        activity = context instanceof Activity ? (Activity) context : getActivity();

        final View contentView = View.inflate(activity, R.layout.settings_dialog_modal, null);

        contentView.findViewById(R.id.btnLogin).setOnClickListener(clickListener);
        // i really hope nobody reads this
        contentView.findViewById(R.id.btnProjectileDysfunction).setOnClickListener(clickListener);

        final AppCompatCheckBox cbDownloadUsername = contentView.findViewById(R.id.cbDownloadUsername);
        final AppCompatCheckBox cbBottomToolbar = contentView.findViewById(R.id.cbBottomToolbar);
        final AppCompatCheckBox cbBottomSnack = contentView.findViewById(R.id.cbBottomSnack);
        final AppCompatCheckBox cbAutoplayVideos = contentView.findViewById(R.id.cbAutoplayVideos);
        final AppCompatCheckBox cbMuteVideos = contentView.findViewById(R.id.cbMuteVideos);

        cbDownloadUsername.setChecked(Main.sharedPreferences.getBoolean(DOWNLOAD_USER_FOLDER, false));
        cbBottomToolbar.setChecked(Main.sharedPreferences.getBoolean(BOTTOM_TOOLBAR, true));
        cbBottomSnack.setChecked(Main.sharedPreferences.getBoolean(BOTTOM_SNACK, true));
        cbAutoplayVideos.setChecked(Main.sharedPreferences.getBoolean(AUTOPLAY_VIDEOS, true));
        cbMuteVideos.setChecked(Main.sharedPreferences.getBoolean(MUTED_VIDEOS, true));

        setupListener(cbDownloadUsername);
        setupListener(cbBottomToolbar);
        setupListener(cbBottomSnack);
        setupListener(cbAutoplayVideos);
        setupListener(cbMuteVideos);

        dialog.setContentView(contentView);

        final CoordinatorLayout.LayoutParams layoutParams = (CoordinatorLayout.LayoutParams)
                ((View) contentView.getParent()).getLayoutParams();
        final CoordinatorLayout.Behavior<?> behavior = layoutParams.getBehavior();
        if (behavior instanceof BottomSheetBehavior) {
            final BottomSheetBehavior<?> sheetBehavior = (BottomSheetBehavior<?>) behavior;
            sheetBehavior.removeBottomSheetCallback(bottomSheet);
            sheetBehavior.addBottomSheetCallback(bottomSheet);
            sheetBehavior.setSaveFlags(BottomSheetBehavior.SAVE_HIDEABLE | BottomSheetBehavior.SAVE_FIT_TO_CONTENTS
                    | BottomSheetBehavior.SAVE_SKIP_COLLAPSED | BottomSheetBehavior.SAVE_PEEK_HEIGHT);
        }

        return dialog;
    }

    private void setupListener(@NonNull final AppCompatCheckBox checkBox) {
        checkBox.setOnCheckedChangeListener(checkChangeListener);
        ((View) checkBox.getParent()).setOnClickListener(clickListener);
    }
}